#include <qimage.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtimer.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <time.h>

#include "ai_player.h"
#include "blast.h"
#include "bomb.h"
#include "item.h"
#include "playcanvas.h"
#include "player.h"
#include "resource.h"
#include "exit.h"

#include <qvaluelist.h>
#include <qpoint.h>

PlayCanvas::PlayCanvas(QObject *parent, const char *name)
	: QCanvas(parent, name)
{
	m_level = NULL;
	m_away = false;
	m_enabled = false;
	m_localenabled = false;
	m_players = 3;

	exit = NULL;
	currPlayer = NULL;

	srand(time(0) + getpid());

	resource = new Resource();

	bombList.setAutoDelete(true);
	blastList.setAutoDelete(true);
	playerList.setAutoDelete(true);
	itemList.setAutoDelete(true);

	startTimer(100);

	monsterTimer = new QTimer(this);
	connect(monsterTimer, SIGNAL(timeout()), this, SLOT(monsterTimeout()));
	monsterTimer->start(500);
}

PlayCanvas::~PlayCanvas()
{
	if (monsterTimer)
		delete monsterTimer;

	playerList.remove(currPlayer);

	currPlayer = 0;
}

void PlayCanvas::setTheme(QString theme)
{
	if(theme.isNull()) return;

	resource->setTheme(theme);

	// reload everything
	if(m_level)
	{
		QPixmap background(resource->path("background"));
		QImage im = background.convertToImage();
		im = im.smoothScale(resource->tilesize() * m_level->width(), resource->tilesize() * m_level->height());
		background.convertFromImage(im);
		setBackgroundPixmap(background);

		/*resize(resource->tilesize() * m_level->width(), resource->tilesize() * m_level->height());*/
		resize(240, 320);
	}

	drawBackground();

	if(exit)
	{
		exit->setSprite(resource->pixmaparray("exit", 3));
		exit->move(exit->x(), exit->y());
	}
	if(currPlayer)
	{
		currPlayer->setSprite(resource->pixmaparray("guy1", 1/*12*/));
		currPlayer->move(currPlayer->x(), currPlayer->y());
	}

	for (Player *player = playerList.first(); player != NULL; player = playerList.next())
	{
		player->setSprite(resource->pixmaparray("guy1", 1/*12*/));
		player->move(player->x(), player->y());
	}
	for (AI_Player *monster = monsterList.first(); monster != NULL; monster = monsterList.next())
	{
		monster->setSprite(resource->pixmaparray("monster1", 2/*13*/));
		monster->move(monster->x(), monster->y());
	}

	for (Blast *blast = blastList.first(); blast != NULL; blast = blastList.next())
	{
		blast->setSprite(resource->pixmaparray("blast", /*10*/4));
		blast->move(blast->x(), blast->y());
	}

	for (Item *item = itemList.first(); item != NULL; item = itemList.next())
	{
		switch(item->type())
		{
			case invalid:
				// nothing
				break;
			case ground:
				break;
				// background
			case wall:
				// background
				break;
			case bridge:
				// background
				break;
			case space:
				// nothing
				break;
			case itemScree:
				item->setSprite(resource->pixmaparray("item_scree", 3));
				break;
			case itemBomb:
				item->setSprite(resource->pixmaparray("item_bomb", 3));
				break;
			case itemLive:
				item->setSprite(resource->pixmaparray("item_live", 3));
				break;
			case itemPower:
				item->setSprite(resource->pixmaparray("item_power", 3));
				break;
		}
		item->move(item->x(), item->y());
	}

	// greyed out
	if(m_enabled == false)
	{
		if(!backgroundPixmap().isNull())
		{
			QImage im = backgroundPixmap().convertToImage();

			for(int j = 0; j < im.height(); j += 2)
				for(int i = 0; i < im.width(); i += 2)
					im.setPixel(i, j, 0);

			QPixmap background;
			background.convertFromImage(im);
			setBackgroundPixmap(background);
		}
	}
}

void PlayCanvas::drawBackground(void)
{
	QPainter painter;
	QString pixmap;
	QPixmap pm;

	if(!m_level) return;
	
	pm = backgroundPixmap();

	if(pm.isNull()) return;

	QPixmap pmWall = resource->pixmap("wall", 0);
	QPixmap pmBridge = resource->pixmap("bridge", 0);
	QPixmap pmGround = resource->pixmap("ground", 0);

	painter.begin(&pm);
	for(int j = 0; j < m_level->height(); j++) {
		for(int i = 0; i < m_level->width(); i++) {
			switch (m_level->tile(i, j)) {
				case wall:
					painter.drawPixmap(resource->tilesize() * i, resource->tilesize() * j, pmWall);
					break;
				case bridge:
					painter.drawPixmap(resource->tilesize() * i, resource->tilesize() * j, pmBridge);
					break;
				case space:
					break;
				case ground:
				default:
					painter.drawPixmap(resource->tilesize() * i, resource->tilesize() * j, pmGround);
					break;
			}

		}
	}

	painter.end();

	setBackgroundPixmap(pm);
}

void PlayCanvas::buildLevel(QString levelfile)
{
	if(levelfile.isNull()) return;

	m_level = new Level();

	m_level->loadLevel(resource->load(levelfile, Resource::level));

	QPixmap background(resource->path("background"));
	QImage im = background.convertToImage();
	im = im.smoothScale(resource->tilesize() * m_level->width(), resource->tilesize() * m_level->height());
	background.convertFromImage(im);
	setBackgroundPixmap(background);

	drawBackground();

	createPlayer();
	createMonster();

	QCanvasSprite *sprite = new QCanvasSprite(resource->pixmaparray("exit", 3), this);
	sprite->setZ(-0.1);
	sprite->setFrame(0);
	exit = new Exit(-1, 0, 0, sprite);

	placeDynamic();

	/*resize(resource->tilesize() * m_level->width(), resource->tilesize() * m_level->height());*/
	/*resize(240, 320);*/
	setAdvancePeriod(1);
	update();
}

void PlayCanvas::createPlayer()
{
	QCanvasSprite *sprite = new QCanvasSprite(resource->pixmaparray("guy1", 1/*12*/), this);
	if(sprite->frameCount() > 2) sprite->setFrame(2);

	currPlayer = new Player(-1, 0, 0, sprite);
	currPlayer->power(3);
	currPlayer->numBombs(4);
	playerList.append(currPlayer);
}

void PlayCanvas::createMonster(int count)
{
	int orig = count;
	if(!count) count = m_players;
	for (int i = 0; i < count; i++) {
		QCanvasPixmapArray *a = resource->pixmaparray("monster1", 2);
		QCanvasSprite *sprite = new QCanvasSprite(a, this);
		if(sprite->frameCount() > 2) sprite->setFrame(2);

		AI_Player *monster = new AI_Player(-1, 0, 0, sprite);
		monsterList.append(monster);
		if(orig)
		{
			QValueList<QPoint> freeCells;
			for (int j = 0; j < m_level->height(); j++)
				for (int i = 0; i < m_level->width(); i++)
					if (m_level->tile(i, j) != invalid && m_level->tile(i, j) != wall && m_level->tile(i, j) != space) {
						QPoint point;
						point.setX(i);
						point.setY(j);
						freeCells.append(point);
					}

			int pos = random(freeCells.count());
			monster->move(freeCells[pos].x(), freeCells[pos].y());
			freeCells.remove(freeCells[pos]);
		}
	}
}

void PlayCanvas::setPlayers(int players)
{
	m_players = players;
}

void PlayCanvas::placeDynamic()
{
	QCanvasSprite *sprite;
	AI_Player *monster;
	Player *player;
	QValueList<QPoint> freeCells;
	int pos, i;

	if(!m_level) return;

	for (int j = 0; j < m_level->height(); j++)
		for (int i = 0; i < m_level->width(); i++)
			if (m_level->tile(i, j) != invalid && m_level->tile(i, j) != wall && m_level->tile(i, j) != space) {
				QPoint point;
				point.setX(i);
				point.setY(j);
				freeCells.append(point);
			}

	int size = (int)(freeCells.count() * 0.25);
	
	// place the scree items
	for (i = 0; i < size; i++) {
		pos = random(freeCells.count());
		
		sprite = new QCanvasSprite(resource->pixmaparray("item_scree", 3), this);
		sprite->setFrame(0);

		Item *item = new Item(-1, freeCells[pos].x(), freeCells[pos].y(), sprite, itemScree);
		itemList.append(item);

		m_level->setTile(freeCells[pos].x(), freeCells[pos].y(), itemScree);

		freeCells.remove(freeCells[pos]);
	}

	// place the other items
	for (i = 0; i < 10; i++) {
		pos = random(freeCells.count());
		int type;

		switch (random(3)) {
			case 0:
				sprite = new QCanvasSprite(resource->pixmaparray("item_bomb", 3), this);
				type = itemBomb;
				break;
			case 1:
				sprite = new QCanvasSprite(resource->pixmaparray("item_power", 3), this);
				type = itemPower;
				break;
			case 2:
				sprite = new QCanvasSprite(resource->pixmaparray("item_live", 3), this);
				type = itemLive;
				break;
		}

		sprite->setFrame(0);

		Item *item = new Item(-1, freeCells[pos].x(), freeCells[pos].y(), sprite, type);
		itemList.append(item);

		m_level->setTile(freeCells[pos].x(), freeCells[pos].y(), type);

		freeCells.remove(freeCells[pos]);
	}
			
	// place the players
	for (player = playerList.first(); player != NULL; player = playerList.next()) {
		pos = random(freeCells.count());
		player->move(freeCells[pos].x(), freeCells[pos].y());
		freeCells.remove(freeCells[pos]);
	}

	// place the monster
	for (monster = monsterList.first(); monster != NULL; monster = monsterList.next()) {
		pos = random(freeCells.count());
		monster->move(freeCells[pos].x(), freeCells[pos].y());
		freeCells.remove(freeCells[pos]);
	}

	// place the exit
	pos = random(freeCells.count());
	exit->move(freeCells[pos].x(), freeCells[pos].y());
	freeCells.remove(freeCells[pos]);
}

bool PlayCanvas::walkableTile(int x, int y)
{
	Bomb *bomb;
	Item *item;
	bool valid = true;

	if(!m_level) return false;

	// do we stay in range
	if ((x >= 0) && (y >= 0) && (x < /*width() / resource->tilesize()*/ m_level->width()) && (y < /*height() / resource->tilesize()*/m_level->height())) {
		// is there something that could block us
		if (m_level->tile(x, y) == wall || m_level->tile(x, y) == invalid || m_level->tile(x, y) == space)
			valid = false;

		if (exit->x() == x && exit->y() == y && exit->sprite()->frame() != 1)
			valid = false;

		if (m_level->tile(x, y) > space) {
			if ((item = findItem(x, y)) != NULL) {
				if (item->sprite()->frame() == 0)
					valid = false;
			}
		}

		if (!bombList.isEmpty()) {
			for (bomb = bombList.first(); bomb != NULL; bomb = bombList.next()) {
				if (bomb->x() == x && bomb->y() == y) {
					valid = false;
					break;
				}
			}
		}
	} else {
		valid = false;
	}

	return valid;
}

void PlayCanvas::action(int event, int option)
{
	int x, y;
	Bomb *bomb;
	QCanvasSprite *bombSprite;

	//kdDebug() << "action " << m_away << endl;

	if (m_away == true)
		return;
	if (m_enabled == false)
		return;

	x = currPlayer->x();
	y = currPlayer->y();
	switch(event)
	{
		case moveEvent:
			switch(option)
			{
				case optLeft:
					x--;
					if(currPlayer->sprite()->frameCount() > 1) currPlayer->sprite()->setFrame(1);
					break;
				case optRight:
					x++;
					if(currPlayer->sprite()->frameCount() > 1) currPlayer->sprite()->setFrame(0);
					break;
				case optUp:
					y--;
					if(currPlayer->sprite()->frameCount() > 3) currPlayer->sprite()->setFrame(3);
					break;
				case optDown:
					if(currPlayer->sprite()->frameCount() > 3) currPlayer->sprite()->setFrame(2);
					y++;
					break;
			}

			if (walkableTile(x, y)) {
				currPlayer->move(x, y);
				checkItem(x, y);
			}

			if (m_level->tile(x, y) == space)
				currPlayer->numLives(0);
			break;

		case fireEvent:
			int valid = 1;

			if (currPlayer->numBombs() == 0)
				valid = 0;

			if (!bombList.isEmpty()) {
				for (bomb = bombList.first(); bomb != NULL; bomb = bombList.next())
					if (bomb->x() == x && bomb->y() == y)
						valid = 0;
			}

			if (valid) {
				bombSprite = new QCanvasSprite(resource->pixmaparray("bomb", /*2*/8), this);
				//bombSprite->setFrame(0);

				bomb = new Bomb(-1, x, y, bombSprite, currPlayer);
				bombList.append(bomb);
				currPlayer->numBombs(currPlayer->numBombs() - 1);
			}
			break;
	}
}

void PlayCanvas::checkItem(int x, int y)
{
	Item *item;

	/* check if player found something */
	if ((item = findItem(x, y)) != NULL) {
		switch (item->type()) {
			case itemBomb:
				currPlayer->numBombs(currPlayer->numBombs() + 1);
				break;
			case itemLive:
				currPlayer->numLives(currPlayer->numLives() + 1);
				break;
			case itemPower:
				currPlayer->power(currPlayer->power() + 1);
				break;
		}
		itemList.remove(item);
		m_level->setTile(x, y, ground);
	}
	
	if (exit->x() == x && exit->y() == y)
		if (exit->isActive())
			emit levelFinished();
}

bool PlayCanvas::placeBlast(int x, int y)
{
	QCanvasSprite *blastSprite;
	Blast *blast;

	if (m_level->tile(x, y) == invalid || m_level->tile(x, y) == wall)
		return false;

	blastSprite = new QCanvasSprite(resource->pixmaparray("blast", /*4*/10), this);
	blastSprite->setFrame(0);

	blast = new Blast(-1, x, y, blastSprite);
	blastList.append(blast);

	if (m_level->tile(x, y) > space)
		return false;
	else
		return true;
}

void PlayCanvas::timerEvent(QTimerEvent *e)
{
	Item *item;

	if(m_enabled == false) return;

	exit->proceed();

	for (Blast *blast = blastList.first(); blast != NULL; blast = blastList.next()) {
		blast->proceed();

		/* check for player, standing in a blast area */
		for (Player *player = playerList.first(); player != NULL; player = playerList.next()) {
			if (player->x() == blast->x() && player->y() == blast->y() && blast->sprite()->frame() == 1)
				player->numLives(player->numLives() - 1);
		}

		for (AI_Player *monster = monsterList.first(); monster != NULL; monster = monsterList.next()) {
			if (monster->x() == blast->x() && monster->y() == blast->y() && blast->sprite()->frame() == 1)
				monster->numLives(monster->numLives() - 1);
		}

		/* check for other bombs, lieing in a blast area */
		for (Bomb *bomb = bombList.first(); bomb != NULL; bomb = bombList.next()) {
			if (bomb->x() == blast->x() && bomb->y() == blast->y())
				bomb->setStep(0);
		}

		/* when exit is hit, generate a new monster */
		if (exit->x() == blast->x() && exit->y() == blast->y() && blast->sprite()->frame() == 1) {
			if (exit->sprite()->frame() == 0)
				exit->sprite()->setFrame(1);
			if (!exit->isActive())
				createMonster(1);
		}

		/* check for other items, lieing in a blast area */
		if ((item = findItem(blast->x(), blast->y())) != NULL) {
			if (blast->sprite()->frame() == 1) {
				if (item->sprite()->frame() != 1)
					if (item->type() == itemScree) {
						m_level->setTile(item->x(), item->y(), ground);
						itemList.remove(item);
					} else
						item->sprite()->setFrame(1);
				else {
					itemList.remove(item);
				}
			}
		}
		if (blast->finished())
			blastList.remove(blast);
	}

	for (Bomb *bomb = bombList.first(); bomb != NULL; bomb = bombList.next())
	{
		bomb->proceed();

		if (bomb->finished()) {
			bomb->player()->numBombs(bomb->player()->numBombs() + 1);

			/* create explosion */
			bool left, right, top, bottom;

			placeBlast(bomb->x(), bomb->y());

			left = right = top = bottom = true;
			for (int i = 1; i < currPlayer->power() + 1; i++) {
				if (right)
					right = placeBlast(bomb->x() + i, bomb->y());
				if (left)
					left = placeBlast(bomb->x() - i, bomb->y());
				if (top)
					top = placeBlast(bomb->x(), bomb->y() + i);
				if (bottom)
					bottom = placeBlast(bomb->x(), bomb->y() - i);
			}

			bombList.remove(bomb);
		}
	}

	for (Player *player = playerList.first(); player != NULL; player = playerList.next()) {
		for (AI_Player *monster = monsterList.first(); monster != NULL; monster = monsterList.next()) {
			if (monster->x()  == player->x() && monster->y() == player->y())
				player->numLives(player->numLives() - 1);
		}

		if (player->numLives() == 0) { /* player died */
			player->sprite()->setFrame(12);
			emit playerKilled();
		}
	}

	for (AI_Player *monster = monsterList.first(); monster != NULL; monster = monsterList.next()) {
		if (monster->numLives() == 0) { /* monster died */
			monster->sprite()->hide();
			monsterList.remove(monster);
		}
	}

	if (monsterList.count() == 0) {
		exit->activate(true);
		for (Item *item = itemList.first(); item != NULL; item = itemList.next()) {
			if(item->type() >= itemBomb)
				item->setHighlighted(true);
		}
	}

	for (Item *item = itemList.first(); item != NULL; item = itemList.next())
		item->proceed();
}

void PlayCanvas::monsterTimeout()
{
	if(m_enabled == false) return;
	if(m_localenabled == false) return;

	for (AI_Player *monster = monsterList.first(); monster != NULL; monster = monsterList.next()) {
		int x = monster->x();
		int y = monster->y();

		if (monster->counter() == 3) {
			if (random(2) == 0) {
				switch (monster->direction()) {
					case AI_Player::Up:
						monster->direction(AI_Player::Down);
						break;
					case AI_Player::Down:
						monster->direction(AI_Player::Up);
						break;
					case AI_Player::Left:
						monster->direction(AI_Player::Right);
						break;
					case AI_Player::Right:
						monster->direction(AI_Player::Left);
						break;
				}
			}
		}

		if (monster->counter() == 4) {
			if (random(2) == 0) {
				switch (monster->direction()) {
					case AI_Player::Up:
						monster->direction(AI_Player::Left);
						break;
					case AI_Player::Down:
						monster->direction(AI_Player::Right);
						break;
					case AI_Player::Left:
						monster->direction(AI_Player::Down);
						break;
					case AI_Player::Right:
						monster->direction(AI_Player::Up);
						break;
				}
			}
			monster->counter(0);
		} else {
			monster->counter(monster->counter() + 1);
		}

		if (monster->direction() == AI_Player::Up) {
			if (walkableTile(x + 1, y)) {
				monster->move(x + 1, y);
				monster->direction(AI_Player::Right);
			} else if (walkableTile(x, y - 1)) {
				monster->move(x, y - 1);
				monster->direction(AI_Player::Up);
			} else if (walkableTile(x - 1, y)) {
				monster->move(x - 1, y);
				monster->direction(AI_Player::Left);
			} else if (walkableTile(x, y + 1)) {
				monster->move(x, y + 1);
				monster->direction(AI_Player::Down);
			}
		} else if (monster->direction() == AI_Player::Down) {
			if (walkableTile(x - 1, y)) {
				monster->move(x - 1, y);
				monster->direction(AI_Player::Left);
			} else if (walkableTile(x, y + 1)) {
				monster->move(x, y + 1);
				monster->direction(AI_Player::Down);
			} else if (walkableTile(x + 1, y)) {
				monster->move(x + 1, y);
				monster->direction(AI_Player::Right);
			} else if (walkableTile(x, y - 1)) {
				monster->move(x, y - 1);
				monster->direction(AI_Player::Up);
			}
		} else if (monster->direction() == AI_Player::Left) {
			if (walkableTile(x, y - 1)) {
				monster->move(x, y - 1);
				monster->direction(AI_Player::Up);
			} else if (walkableTile(x - 1, y)) {
				monster->move(x - 1, y);
				monster->direction(AI_Player::Left);
			} else if (walkableTile(x, y + 1)) {
				monster->move(x, y + 1);
				monster->direction(AI_Player::Down);
			} else if (walkableTile(x + 1, y)) {
				monster->move(x + 1, y);
				monster->direction(AI_Player::Right);
			}
		} else if (monster->direction() == AI_Player::Right) {
			if (walkableTile(x, y + 1)) {
				monster->move(x, y + 1);
				monster->direction(AI_Player::Down);
			} else if (walkableTile(x + 1, y)) {
				monster->move(x + 1, y);
				monster->direction(AI_Player::Right);
			} else if (walkableTile(x, y - 1)) {
				monster->move(x, y - 1);
				monster->direction(AI_Player::Up);
			} else if (walkableTile(x - 1, y)) {
				monster->move(x - 1, y);
				monster->direction(AI_Player::Left);
			}
		}
	}
}

int PlayCanvas::random(int max)
{
	if(!max) return 0;
	return (rand() % max);
}

Item *PlayCanvas::findItem(int x, int y)
{
	Item *item;

	for (item = itemList.first(); item != NULL; item = itemList.next())
		if (item->x() == x && item->y() == y)
			return item;
	
	return NULL;
}

Level *PlayCanvas::level()
{
	return m_level;
}

void PlayCanvas::enable(bool enabled)
{
	m_enabled = enabled;
	setTheme(resource->theme());
}

void PlayCanvas::enableLocal(bool enabled)
{
	m_localenabled = enabled;
}

