#ifndef PLAYCANVAS_H
#define PLAYCANVAS_H

#include <qcanvas.h>
#include <qlist.h>

#include "level.h"

class QPixmap;
class QTimer;
class Bomb;
class Player;
class Blast;
class Item;
class Exit;
class AI_Player;
class Resource;

class PlayCanvas : public QCanvas
{
	Q_OBJECT
public:
	PlayCanvas(QObject *parent = NULL, const char *name = NULL);
	~PlayCanvas();

	enum Events
	{
		moveEvent,
		fireEvent
	};
	enum Options
	{
		optNone,
		optLeft,
		optRight,
		optDown,
		optUp
	};

	enum TileTypes
	{
		invalid,
		ground,
		wall,
		bridge,
		space,
		itemScree,
		itemBomb,
		itemLive,
		itemPower
	};

	void buildLevel(QString levelfile);
	void drawBackground(void);
	void placeDynamic(void);
	void createPlayer();
	void createMonster(int count = 0);
	bool placeBlast(int x, int y);
	void checkItem(int x, int y);
	void action(int event, int option);
	bool walkableTile(int x, int y);
	void setTheme(QString theme);
	void setPlayers(int players);

	void enable(bool enabled);
	void enableLocal(bool enabled);

	Level *level();

public slots:
	void monsterTimeout();

signals:
	void playerKilled();
	void levelFinished();

protected:
	void timerEvent(QTimerEvent *e);

private:
	int random(int max);
	Item *findItem(int x, int y);
	Level *m_level;

	QList<Bomb> bombList;
	QList<Item> itemList;
	QList<Blast> blastList;
	QList<Player> playerList;
	QList<AI_Player> monsterList;

	QTimer *monsterTimer;

	Player *currPlayer;
	Resource *resource;
	Exit *exit;
	bool m_away;
	int m_players;
	bool m_enabled;
	bool m_localenabled;
};

#endif
