#include "resource.h"
#include <stdio.h>
#include <qimage.h>

QString s_theme;
int s_tilesize;
QMap<QString, QCanvasPixmapArray*> Resource::m_pixmaparrayhash;
QMap<QString, QPixmap*> Resource::m_pixmaphash;

void Resource::setTheme(QString theme)
{
	clean();
	s_theme = theme;
}

void Resource::setTilesize(int tilesize)
{
	clean();
	s_tilesize = tilesize;
}

QString Resource::path(QString item)
{
	if(item == "item_bomb") return Resource::load("item_bomb%1.png", Resource::pic);
	if(item == "item_live") return Resource::load("item_live%1.png", Resource::pic);
	if(item == "item_power") return Resource::load("item_power%1.png", Resource::pic);
	if(item == "item_scree") return Resource::load("item_scree%1.png", Resource::pic);

	if(item == "ground") return Resource::load("ground.png", Resource::pic);
	if(item == "wall") return Resource::load("wall.png", Resource::pic);
	if(item == "bridge") return Resource::load("bridge.png", Resource::pic);
	if(item == "exit") return Resource::load("item_exit%1.png", Resource::pic);

	if(item == "bomb") return Resource::load("bomb%1.png", Resource::pic);
	if(item == "blast") return Resource::load("blast%1.png", Resource::pic);

	if(item == "background") return Resource::load("bg.png", Resource::pic);

	if(item == "guy1") return Resource::load("guy%1.png", Resource::pic);
	if(item == "guy2") return Resource::load("guy2%1.png", Resource::pic);
	if(item == "guy3") return Resource::load("guy3%1.png", Resource::pic);
	if(item == "guy4") return Resource::load("guy4%1.png", Resource::pic);
	if(item == "monster1") return Resource::load("monster%1.png", Resource::pic);
	if(item == "monster2") return Resource::load("monster2%1.png", Resource::pic);

	fprintf(stderr, "Resource: '%s' unknown\n", item.latin1());

	return QString::null;
}

QString Resource::load(QString item, int type)
{
	switch(type)
	{
		case pic:
			return QString("./pics/theme-") + s_theme + "/" + item;
			break;
		case level:
			return QString("./levels/") + item + ".txt";
			break;
	}
}

QPixmap Resource::pixmap(QString item, int count)
{
	if(m_pixmaphash[item]) return *(m_pixmaphash[item]);

	QPixmap pic(path(item));
	QImage im = pic.convertToImage();
	QImage im2 = im.smoothScale(tilesize(), tilesize());
	pic.convertFromImage(im2);

	m_pixmaphash[item] = new QPixmap(pic);

	return pic;
}

QCanvasPixmapArray *Resource::pixmaparray(QString item, int count)
{
	if(m_pixmaparrayhash[item]) return m_pixmaparrayhash[item];

	int count2 = 0;

	if(s_theme == "desert")
	{
		if(item == "bomb") count2 = 8;
		if(item == "blast") count2 = 10;
		if(item == "guy1") count2 = 2;
		if(item == "monster1") count2 = 2;
	}
	else if(s_theme == "standard")
	{
		if(item == "bomb") count2 = 8;
		if(item == "blast") count2 = 10;
		if(item == "guy1") count2 = 2;
		if(item == "monster1") count2 = 2;
	}
	else if(s_theme == "nerdworld")
	{
		if(item == "bomb") count2 = 8;
		if(item == "blast") count2 = 10;
		if(item == "guy1") count2 = 4;
		if(item == "monster1") count2 = 2;
	}

	if(count2 > 0)
	{
		count = count2;
	}

	QCanvasPixmapArray *picar = new QCanvasPixmapArray(path(item), count);
	for(int i = 0; i < picar->count(); i++)
	{
		QCanvasPixmap *pic = picar->image(i);
		QImage im = pic->convertToImage();
		QImage im2 = im.smoothScale(tilesize(), tilesize());
		pic->convertFromImage(im2);
	}

	m_pixmaparrayhash[item] = picar;

	return picar;
}

void Resource::clean()
{
	m_pixmaparrayhash.clear();
	m_pixmaphash.clear();
}

int Resource::tilesize()
{
	return s_tilesize;
}

QString Resource::theme()
{
	return s_theme;
}

