#include "select.h"
#include "playground.h"
#include "playcanvas.h"
#include "resource.h"

#include <qlayout.h>
#include <qframe.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qcombobox.h>

#include <stdio.h>

Select::Select(QWidget *parent, const char *name)
: QDialog(parent, name, TRUE)
{
	QVBoxLayout *vbox;
	QHBoxLayout *hbox, *hbox2;
	QPushButton *ok, *cancel;
	QLabel *desclevel, *desctheme, *descplayers;

	m_playground = NULL;
	m_playcanvas = NULL;

	desclevel = new QLabel("Level to play", this);
	desctheme = new QLabel("Theme to use", this);
	descplayers = new QLabel("Number of AI players", this);

	combolevel = new QComboBox(this);

	combolevel->insertItem("Test");
	m_map["Test"] = "zaurus_12_13";

	combotheme = new QComboBox(this);

	combotheme->insertItem("Standard");
	m_map2["Standard"] = "standard";
	combotheme->insertItem("Desert");
	m_map2["Desert"] = "desert";
	combotheme->insertItem("Nerdworld");
	m_map2["Nerdworld"] = "nerdworld";

	comboplayers = new QComboBox(this);

	preview = new QFrame(this);
	preview->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	preview->setFixedSize(12 * 6, 13 * 6);
	
	xlayout = new QVBoxLayout(preview, 2);

	ok = new QPushButton("Select", this);
	cancel = new QPushButton("Cancel", this);

	vbox = new QVBoxLayout(this, 5);
	vbox->add(desclevel);
	vbox->add(combolevel);
	vbox->add(desctheme);
	vbox->add(combotheme);
	vbox->add(descplayers);
	vbox->add(comboplayers);
	hbox2 = new QHBoxLayout(vbox, 5);
	hbox2->addStretch(1);
	hbox2->add(preview);
	hbox2->addStretch(1);
	hbox = new QHBoxLayout(vbox, 5);
	hbox->add(ok);
	hbox->add(cancel);

	connect(comboplayers, SIGNAL(activated(int)), SLOT(minimap()));
	connect(combolevel, SIGNAL(activated(int)), SLOT(minimap()));
	connect(combotheme, SIGNAL(activated(int)), SLOT(minimap()));
	connect(ok, SIGNAL(clicked()), SLOT(slotAccept()));
	connect(cancel, SIGNAL(clicked()), SLOT(reject()));

	setBots(4);

	setCaption("Select a level");

	minimap();
}

Select::~Select()
{
}

void Select::setBots(int bots)
{
	comboplayers->clear();
	for(int i = 0; i < bots; i++)
	{
		comboplayers->insertItem(QString("%1").arg(i + 1));
		comboplayers->setCurrentItem(i);
	}
}

void Select::setBotsReadOnly()
{
	comboplayers->setEnabled(false);
}

void Select::slotAccept()
{
	if(m_playground) delete m_playground;
	if(m_playcanvas) delete m_playcanvas;

	accept();
}

void Select::minimap()
{
	if(m_playground) delete m_playground;
	if(m_playcanvas) delete m_playcanvas;

	int oldsize = Resource::tilesize();
	Resource::setTilesize(6);
	m_playcanvas = new PlayCanvas(preview);
	m_playcanvas->setTheme(theme());
	m_playcanvas->setPlayers(players());
	m_playcanvas->buildLevel(level());
	Resource::setTilesize(oldsize);

	m_playground = new Playground(m_playcanvas, preview);
	m_playground->resize(preview->width() - 4, preview->height() - 4);
	xlayout->add(m_playground);
}

QString Select::level()
{
	return m_map[combolevel->currentText()];
}

QString Select::theme()
{
	return m_map2[combotheme->currentText()];
}

int Select::players()
{
	return comboplayers->currentText().toInt();
}

