#include "toplevel.h"
#include "playground.h"
#include "playcanvas.h"
#include "network.h"
#include "qmenubar.h"
#include "qpopupmenu.h"
#include "splash.h"
#include "select.h"
#include "level.h"
#include "resource.h"
#include "protocol.h"

#include <qapplication.h>
#include <qmessagebox.h>

#include <qsocketdevice.h>

TopLevel::TopLevel()
	: QMainWindow()
{
	QPopupMenu *gamemenu, *displaymenu;

	Resource::setTheme("desert");
	Resource::setTilesize(20);

	playground = NULL;
	canvas = NULL;
	splash = new Splash(this);
	setCentralWidget(splash);

	gamemenu = new QPopupMenu(this);
	gamemenu->insertItem("New", game_new);
	gamemenu->insertSeparator();
	gamemenu->insertItem("Exit", game_exit);

	displaymenu = new QPopupMenu(this);
	displaymenu->insertItem("Larger", display_larger);
	displaymenu->insertItem("Smaller", display_smaller);

	menuBar()->insertItem("Game", gamemenu);
	menuBar()->insertItem("Display", displaymenu);

	connect(gamemenu, SIGNAL(activated(int)), SLOT(slotMenu(int)));
	connect(displaymenu, SIGNAL(activated(int)), SLOT(slotMenu(int)));

	setCaption("Opie-Kamikaze");
	showMaximized();
}

TopLevel::~TopLevel()
{
}

void TopLevel::key(int keycode)
{
	int gen;

	if(!canvas) return;

	switch(keycode)
	{
		case Playground::KeyLeft:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optLeft);
			break;
		case Playground::KeyRight:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optRight);
			break;
		case Playground::KeyUp:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optUp);
			break;
		case Playground::KeyDown:
			canvas->action(PlayCanvas::moveEvent, PlayCanvas::optDown);
			break;
		case Playground::KeyFire:
			canvas->action(PlayCanvas::fireEvent, PlayCanvas::optNone);
			break;
		case Playground::KeyZoomOut:
			scale(-5);
			break;
		case Playground::KeyZoomIn:
			scale(5);
			break;
	}
}

void TopLevel::newGame(QString level, QString theme, int players)
{
	if (!canvas && !playground) {
		canvas = new PlayCanvas(this);

		playground = new Playground(canvas, this);
		connect(playground, SIGNAL(key(int)), SLOT(key(int)));
		connect(canvas, SIGNAL(playerKilled()), SLOT(slotEndGame()));
		connect(canvas, SIGNAL(levelFinished()), SLOT(slotEndGame()));

		canvas->setTheme(theme);
		canvas->setPlayers(players);
		canvas->buildLevel(level);
		playground->setFocus();

		splash->hide();
		playground->show();
		setCentralWidget(playground);

		scale(0);
	}
}

void TopLevel::selectNewGame(int bots)
{
	int ret;
	Select sel;

	sel.setBots(bots);
	ret = sel.exec();

	if(ret == QDialog::Accepted)
	{
		newGame(sel.level(), sel.theme(), sel.players());
		canvas->enable(true);
		canvas->enableLocal(true);
	}
}

void TopLevel::scale(int difference)
{
	Resource::setTilesize(Resource::tilesize() + difference);
	if(!canvas)return;
	canvas->setTheme(Resource::theme());
	/*setFixedSize(canvas->level()->width() * Resource::tilesize(),
		canvas->level()->height() * Resource::tilesize() + menuBar()->height());*/
}

void TopLevel::slotMenu(int id)
{
	switch(id)
	{
		case game_new:
			selectNewGame(8);
			break;
		case game_exit:
			slotEndGame();
			close();
			break;
		case display_larger:
			scale(5);
			break;
		case display_smaller:
			scale(-5);
			break;
	}
}

void TopLevel::setTheme(QString theme)
{
	if(canvas) canvas->setTheme(theme);
}

void TopLevel::slotEndGame()
{
	setCentralWidget(splash);
	splash->show();
	setFixedSize(splash->autowidth(), splash->autoheight());

	if (playground)
		delete playground;
	if (canvas)
		delete canvas;

	playground = NULL;
	canvas = NULL;
}

void TopLevel::keyPressEvent(QKeyEvent *e)
{
	switch(e->key())
	{
		case Qt::Key_Down:
			key(Playground::KeyDown);
			break;
		case Qt::Key_Up:
			key(Playground::KeyUp);
			break;
		case Qt::Key_Left:
			key(Playground::KeyLeft);
			break;
		case Qt::Key_Right:
			key(Playground::KeyRight);
			break;
		case Qt::Key_Space:
			key(Playground::KeyFire);
			break;
		case Qt::Key_Minus:
			key(Playground::KeyZoomOut);
			break;
		case Qt::Key_Plus:
			key(Playground::KeyZoomIn);
			break;
	}
}

